#include <stdio.h>
#include <stdlib.h>
#include "headers/bmp_writer.h"

enum write_status to_bmp(FILE *out, struct image const *img) {
    struct bmp_header header;

    header.bfType = 0x4D42;
    header.bfileSize = sizeof(struct bmp_header) + img->width * img->height * sizeof(struct pixel);
    header.bfReserved = 0;
    header.bOffBits = sizeof(struct bmp_header);
    header.biSize = 40;
    header.biWidth = img->width;
    header.biHeight = img->height;
    header.biPlanes = 1;
    header.biBitCount = 24;
    header.biCompression = 0;
    header.biSizeImage = img->width * img->height * sizeof(struct pixel);
    header.biXPelsPerMeter = 0;
    header.biYPelsPerMeter = 0;
    header.biClrUsed = 0;
    header.biClrImportant = 0;

    if (fwrite(&header, sizeof(struct bmp_header), 1, out) != 1) {
        return WRITE_HEADER_ERROR;
    }

    for (size_t i = 0; i < img->height; i++) {
        if (fwrite(&img->data[i * img->width], sizeof(struct pixel), img->width, out) != img->width) {
            return WRITE_ERROR;
        }
        int row_size = ((header.biBitCount * header.biWidth + 31) / 32) * 4;
        if (row_size > img->width * sizeof(struct pixel)) {
            uint8_t padding[3] = {0, 0, 0};
            fwrite(padding, 1, row_size - img->width * sizeof(struct pixel), out);
        }
    }

    return WRITE_OK;
}